@@ -2,16 +2,14 @@
 //!
 //! The set of valid values for FTP commands
 
+use super::Status;
 use std::convert::From;
+use std::fmt;
 use thiserror::Error;
 
-/// ## Result
-///
 /// A shorthand for a Result whose error type is always an FtpError.
-pub type Result<T> = std::result::Result<T, FtpError>;
+pub type FtpResult<T> = std::result::Result<T, FtpError>;
 
-/// ## FtpError
-///
 /// `FtpError` is a library-global error type to describe the different kinds of
 /// errors that might occur while using FTP.
 #[derive(Debug, Error)]
@@ -36,18 +34,13 @@ pub enum FtpError {
     InvalidAddress(std::net::AddrParseError),
 }
 
-/// ## Response
-///
 /// Defines a response from the ftp server
 #[derive(Clone, Debug, Error)]
-#[error("[{code}] {body}")]
 pub struct Response {
-    pub code: u32,
+    pub status: Status,
     pub body: String,
 }
 
-/// ## FormatControl
-///
 /// Text Format Control used in `TYPE` command
 #[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord)]
 pub enum FormatControl {
@@ -61,8 +54,6 @@ pub enum FormatControl {
     Asa,
 }
 
-/// ## FileType
-///
 /// File Type used in `TYPE` command
 #[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord)]
 pub enum FileType {
@@ -78,13 +69,24 @@ pub enum FileType {
     Local(u8),
 }
 
+/// Connection mode for data channel
+#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+pub enum Mode {
+    Passive,
+    Active,
+}
+
+impl fmt::Display for Response {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        write!(f, "[{}] {}", self.status.code(), self.body)
+    }
+}
+
 impl Response {
-    /// ### new
-    ///
     /// Instantiates a new `Response`
-    pub fn new<S: AsRef<str>>(code: u32, body: S) -> Self {
+    pub fn new<S: AsRef<str>>(status: Status, body: S) -> Self {
         Self {
-            code,
+            status,
             body: body.as_ref().to_string(),
         }
     }
@@ -134,10 +136,10 @@ mod test {
             "Secure error: omar"
         );
         assert_eq!(
-            FtpError::UnexpectedResponse(Response::new(0, "error"))
+            FtpError::UnexpectedResponse(Response::new(Status::ExceededStorage, "error"))
                 .to_string()
                 .as_str(),
-            "Invalid response: [0] error"
+            "Invalid response: [552] error"
         );
         assert_eq!(
             FtpError::BadResponse.to_string().as_str(),
@@ -147,14 +149,17 @@ mod test {
 
     #[test]
     fn response() {
-        let response: Response = Response::new(0, "error");
-        assert_eq!(response.code, 0);
+        let response: Response = Response::new(Status::AboutToSend, "error");
+        assert_eq!(response.status, Status::AboutToSend);
         assert_eq!(response.body.as_str(), "error");
     }
 
     #[test]
     fn fmt_response() {
-        let response: Response = Response::new(550, "Can't create directory: File exists");
+        let response: Response = Response::new(
+            Status::FileUnavailable,
+            "Can't create directory: File exists",
+        );
         assert_eq!(
             response.to_string().as_str(),
             "[550] Can't create directory: File exists"
